@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2013 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -33,6 +34,7 @@
 #include <time.h>
 
 #include "folder.h"
+#include "folder_item_prefs.h"
 #include "mh.h"
 #include "procmsg.h"
 #include "procheader.h"
@@ -75,21 +77,21 @@ static gint     mh_add_msg		(Folder		*folder,
 static gint     mh_add_msgs		(Folder		*folder,
 					 FolderItem	*dest,
 					 GSList		*file_list,
-					 GRelation 	*relation);
+					 GHashTable 	*relation);
 static gint     mh_copy_msg		(Folder		*folder,
 					 FolderItem	*dest,
 					 MsgInfo	*msginfo);
 static gint	mh_copy_msgs		(Folder 	*folder, 
 					 FolderItem 	*dest, 
 					 MsgInfoList 	*msglist, 
-			 		 GRelation 	*relation);
+			 		 GHashTable 	*relation);
 static gint     mh_remove_msg		(Folder		*folder,
 					 FolderItem	*item,
 					 gint 		 num);
 static gint 	mh_remove_msgs		(Folder 	*folder, 
 					 FolderItem 	*item, 
 		    			 MsgInfoList 	*msglist, 
-					 GRelation 	*relation);
+					 GHashTable 	*relation);
 static gint     mh_remove_all_msg	(Folder		*folder,
 					 FolderItem	*item);
 static gboolean mh_is_msg_changed	(Folder		*folder,
@@ -134,7 +136,7 @@ static int mh_item_close		(Folder		*folder,
 					 FolderItem	*item);
 #if 0
 static gint mh_get_flags		(Folder *folder, FolderItem *item,
-                           		 MsgInfoList *msginfo_list, GRelation *msgflags);
+                           		 MsgInfoList *msginfo_list, GHashTable *msgflags);
 #endif
 static void mh_write_sequences		(FolderItem 	*item, gboolean remove_unseen);
 
@@ -146,6 +148,7 @@ FolderClass *mh_get_class(void)
 		mh_class.type = F_MH;
 		mh_class.idstr = "mh";
 		mh_class.uistr = "MH";
+		mh_class.supports_server_search = FALSE;
 		
 		/* Folder functions */
 		mh_class.new_folder = mh_folder_new;
@@ -173,6 +176,7 @@ FolderClass *mh_get_class(void)
 		mh_class.add_msgs = mh_add_msgs;
 		mh_class.copy_msg = mh_copy_msg;
 		mh_class.copy_msgs = mh_copy_msgs;
+		mh_class.search_msgs = folder_item_search_msgs_local;
 		mh_class.remove_msg = mh_remove_msg;
 		mh_class.remove_msgs = mh_remove_msgs;
 		mh_class.remove_all_msg = mh_remove_all_msg;
@@ -210,7 +214,7 @@ gboolean mh_scan_required(Folder *folder, FolderItem *item)
 	struct stat s;
 
 	path = folder_item_get_path(item);
-	g_return_val_if_fail(path != NULL, FALSE);
+	cm_return_val_if_fail(path != NULL, FALSE);
 
 	if (g_stat(path, &s) < 0) {
 		FILE_OP_ERROR(path, "stat");
@@ -244,12 +248,12 @@ static void mh_get_last_num(Folder *folder, FolderItem *item)
 	gint max = 0;
 	gint num;
 
-	g_return_if_fail(item != NULL);
+	cm_return_if_fail(item != NULL);
 
 	debug_print("mh_get_last_num(): Scanning %s ...\n", item->path?item->path:"(null)");
 
 	path = folder_item_get_path(item);
-	g_return_if_fail(path != NULL);
+	cm_return_if_fail(path != NULL);
 	if (change_dir(path) < 0) {
 		g_free(path);
 		return;
@@ -284,14 +288,14 @@ gint mh_get_num_list(Folder *folder, FolderItem *item, GSList **list, gboolean *
 	struct dirent *d;
 	gint num, nummsgs = 0;
 
-	g_return_val_if_fail(item != NULL, -1);
+	cm_return_val_if_fail(item != NULL, -1);
 
 	debug_print("mh_get_num_list(): Scanning %s ...\n", item->path?item->path:"(null)");
 
 	*old_uids_valid = TRUE;
 
 	path = folder_item_get_path(item);
-	g_return_val_if_fail(path != NULL, -1);
+	cm_return_val_if_fail(path != NULL, -1);
 	if (change_dir(path) < 0) {
 		g_free(path);
 		return -1;
@@ -320,8 +324,8 @@ static gchar *mh_fetch_msg(Folder *folder, FolderItem *item, gint num)
 	gchar *path;
 	gchar *file;
 
-	g_return_val_if_fail(item != NULL, NULL);
-	g_return_val_if_fail(num > 0, NULL);
+	cm_return_val_if_fail(item != NULL, NULL);
+	cm_return_val_if_fail(num > 0, NULL);
 
 	path = folder_item_get_path(item);
 	file = g_strconcat(path, G_DIR_SEPARATOR_S, itos(num), NULL);
@@ -340,7 +344,7 @@ static MsgInfo *mh_get_msginfo(Folder *folder, FolderItem *item, gint num)
 	MsgInfo *msginfo;
 	gchar *file;
 
-	g_return_val_if_fail(item != NULL, NULL);
+	cm_return_val_if_fail(item != NULL, NULL);
 	if (num <= 0)
 		return NULL;
 
@@ -362,7 +366,7 @@ static gchar *mh_get_new_msg_filename(FolderItem *dest)
 	gchar *destpath;
 
 	destpath = folder_item_get_path(dest);
-	g_return_val_if_fail(destpath != NULL, NULL);
+	cm_return_val_if_fail(destpath != NULL, NULL);
 
 	if (!is_dir_exist(destpath))
 		make_dir_hier(destpath);
@@ -388,7 +392,7 @@ static gint mh_add_msg(Folder *folder, FolderItem *dest, const gchar *file, MsgF
 	GSList file_list;
 	MsgFileInfo fileinfo;
 
-	g_return_val_if_fail(file != NULL, -1);
+	cm_return_val_if_fail(file != NULL, -1);
 
 	fileinfo.msginfo = NULL;
 	fileinfo.file = (gchar *)file;
@@ -401,14 +405,14 @@ static gint mh_add_msg(Folder *folder, FolderItem *dest, const gchar *file, MsgF
 } 
  
 static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list, 
-                 GRelation *relation)
+                 GHashTable *relation)
 { 
 	gchar *destfile;
 	GSList *cur;
 	MsgFileInfo *fileinfo;
 
-	g_return_val_if_fail(dest != NULL, -1);
-	g_return_val_if_fail(file_list != NULL, -1);
+	cm_return_val_if_fail(dest != NULL, -1);
+	cm_return_val_if_fail(file_list != NULL, -1);
 
 	if (dest->last_num < 0) {
 		mh_get_last_num(folder, dest);
@@ -435,7 +439,7 @@ static gint mh_add_msgs(Folder *folder, FolderItem *dest, GSList *file_list,
 #endif
 
 		if (relation != NULL)
-			g_relation_insert(relation, fileinfo, GINT_TO_POINTER(dest->last_num + 1));
+			g_hash_table_insert(relation, fileinfo, GINT_TO_POINTER(dest->last_num + 1));
 		g_free(destfile);
 		dest->last_num++;
 	}
@@ -447,7 +451,7 @@ static gint mh_copy_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
 {
 	GSList msglist;
 
-	g_return_val_if_fail(msginfo != NULL, -1);
+	cm_return_val_if_fail(msginfo != NULL, -1);
 
 	msglist.data = msginfo;
 	msglist.next = NULL;
@@ -456,14 +460,13 @@ static gint mh_copy_msg(Folder *folder, FolderItem *dest, MsgInfo *msginfo)
 }
 
 static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist, 
-			 GRelation *relation)
+			 GHashTable *relation)
 {
 	gboolean dest_need_scan = FALSE;
 	gboolean src_need_scan = FALSE;
 	FolderItem *src = NULL;
 	gchar *srcfile;
 	gchar *destfile;
-	gint filemode = 0;
 	FolderItemPrefs *prefs;
 	MsgInfo *msginfo = NULL;
 	MsgInfoList *cur = NULL;
@@ -473,12 +476,12 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 	time_t last_dest_mtime = (time_t)0;
 	time_t last_src_mtime = (time_t)0;
 
-	g_return_val_if_fail(dest != NULL, -1);
-	g_return_val_if_fail(msglist != NULL, -1);
+	cm_return_val_if_fail(dest != NULL, -1);
+	cm_return_val_if_fail(msglist != NULL, -1);
 	
 	msginfo = (MsgInfo *)msglist->data;
 
-	g_return_val_if_fail(msginfo != NULL, -1);
+	cm_return_val_if_fail(msginfo != NULL, -1);
 
 	if (msginfo->folder == dest) {
 		g_warning("the src folder is identical to the dest.\n");
@@ -572,14 +575,13 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 		if (prefs && prefs->enable_folder_chmod && prefs->folder_chmod) {
 			if (chmod(destfile, prefs->folder_chmod) < 0)
 				FILE_OP_ERROR(destfile, "chmod");
-
-			/* for mark file */
-			filemode = prefs->folder_chmod;
-			if (filemode & S_IRGRP) filemode |= S_IWGRP;
-			if (filemode & S_IROTH) filemode |= S_IWOTH;
 		}
-		if (relation)
-			g_relation_insert(relation, msginfo, GINT_TO_POINTER(dest->last_num+1));
+		if (relation) {
+			if (g_hash_table_lookup(relation, msginfo) != NULL)
+				g_warning("already in : %p", msginfo);
+			
+			g_hash_table_insert(relation, msginfo, GINT_TO_POINTER(dest->last_num+1));
+		}
 		g_free(srcfile);
 		g_free(destfile);
 		dest->last_num++;
@@ -618,10 +620,10 @@ static gint mh_remove_msg(Folder *folder, FolderItem *item, gint num)
 	time_t last_mtime = (time_t)0;
 	gchar *file;
 
-	g_return_val_if_fail(item != NULL, -1);
+	cm_return_val_if_fail(item != NULL, -1);
 
 	file = mh_fetch_msg(folder, item, num);
-	g_return_val_if_fail(file != NULL, -1);
+	cm_return_val_if_fail(file != NULL, -1);
 
 	need_scan = mh_scan_required(folder, item);
 	last_mtime = item->mtime;
@@ -640,7 +642,7 @@ static gint mh_remove_msg(Folder *folder, FolderItem *item, gint num)
 }
 
 static gint mh_remove_msgs(Folder *folder, FolderItem *item, 
-		    MsgInfoList *msglist, GRelation *relation)
+		    MsgInfoList *msglist, GHashTable *relation)
 {
 	gboolean need_scan = FALSE;
 	gchar *path, *file;
@@ -648,7 +650,7 @@ static gint mh_remove_msgs(Folder *folder, FolderItem *item,
 	MsgInfoList *cur;
 	gint total = 0, curnum = 0;
 
-	g_return_val_if_fail(item != NULL, -1);
+	cm_return_val_if_fail(item != NULL, -1);
 
 	path = folder_item_get_path(item);
 	
@@ -704,10 +706,10 @@ static gint mh_remove_all_msg(Folder *folder, FolderItem *item)
 	gchar *path;
 	gint val;
 
-	g_return_val_if_fail(item != NULL, -1);
+	cm_return_val_if_fail(item != NULL, -1);
 
 	path = folder_item_get_path(item);
-	g_return_val_if_fail(path != NULL, -1);
+	cm_return_val_if_fail(path != NULL, -1);
 	val = remove_all_numbered_files(path);
 	g_free(path);
 
@@ -736,7 +738,7 @@ static gint mh_scan_tree(Folder *folder)
 	FolderItem *item;
 	gchar *rootpath;
 
-	g_return_val_if_fail(folder != NULL, -1);
+	cm_return_val_if_fail(folder != NULL, -1);
 
 	if (!folder->node) {
 		item = folder_item_new(folder, folder->name, NULL);
@@ -774,19 +776,50 @@ static gint mh_scan_tree(Folder *folder)
 
 static gint mh_create_tree(Folder *folder)
 {
-	gchar *rootpath;
+	gchar *rootpath, *f;
 
-	g_return_val_if_fail(folder != NULL, -1);
+	cm_return_val_if_fail(folder != NULL, -1);
 
 	CHDIR_RETURN_VAL_IF_FAIL(get_mail_base_dir(), -1);
 	rootpath = LOCAL_FOLDER(folder)->rootpath;
 	MAKE_DIR_IF_NOT_EXIST(rootpath);
 	CHDIR_RETURN_VAL_IF_FAIL(rootpath, -1);
-	MAKE_DIR_IF_NOT_EXIST(INBOX_DIR);
-	MAKE_DIR_IF_NOT_EXIST(OUTBOX_DIR);
-	MAKE_DIR_IF_NOT_EXIST(QUEUE_DIR);
-	MAKE_DIR_IF_NOT_EXIST(DRAFT_DIR);
-	MAKE_DIR_IF_NOT_EXIST(TRASH_DIR);
+
+	/* Create special directories as needed */
+	if (folder->inbox != NULL &&
+			folder->inbox->path != NULL)
+		f = folder->inbox->path;
+	else
+		f = INBOX_DIR;
+	MAKE_DIR_IF_NOT_EXIST(f);
+
+	if (folder->outbox != NULL &&
+			folder->outbox->path != NULL)
+		f = folder->outbox->path;
+	else
+		f = OUTBOX_DIR;
+	MAKE_DIR_IF_NOT_EXIST(f);
+
+	if (folder->draft != NULL &&
+			folder->draft->path != NULL)
+		f = folder->draft->path;
+	else
+		f = DRAFT_DIR;
+	MAKE_DIR_IF_NOT_EXIST(f);
+
+	if (folder->queue != NULL &&
+			folder->queue->path != NULL)
+		f = folder->queue->path;
+	else
+		f = QUEUE_DIR;
+	MAKE_DIR_IF_NOT_EXIST(f);
+
+	if (folder->trash != NULL &&
+			folder->trash->path != NULL)
+		f = folder->trash->path;
+	else
+		f = TRASH_DIR;
+	MAKE_DIR_IF_NOT_EXIST(f);
 
 	return 0;
 }
@@ -797,11 +830,11 @@ static gchar *mh_item_get_path(Folder *folder, FolderItem *item)
 {
 	gchar *folder_path, *path;
 	gchar *real_path;
-	g_return_val_if_fail(folder != NULL, NULL);
-	g_return_val_if_fail(item != NULL, NULL);
+	cm_return_val_if_fail(folder != NULL, NULL);
+	cm_return_val_if_fail(item != NULL, NULL);
 
 	folder_path = g_strdup(LOCAL_FOLDER(folder)->rootpath);
-	g_return_val_if_fail(folder_path != NULL, NULL);
+	cm_return_val_if_fail(folder_path != NULL, NULL);
 
         /* FIXME: [W32] The code below does not correctly merge
            relative filenames; there should be a function to handle
@@ -826,7 +859,7 @@ static gchar *mh_item_get_path(Folder *folder, FolderItem *item)
 	if (!is_dir_exist(real_path) && is_dir_exist(path)) {
 		/* mmh, older version did put utf8 filenames instead of
 		 * the correct encoding */
-		rename(path, real_path);
+		g_rename(path, real_path);
 		folder_item_scan(item);
 	}
 
@@ -843,9 +876,9 @@ static FolderItem *mh_create_folder(Folder *folder, FolderItem *parent,
 	gchar *mh_sequences_filename;
 	FILE *mh_sequences_file;
 
-	g_return_val_if_fail(folder != NULL, NULL);
-	g_return_val_if_fail(parent != NULL, NULL);
-	g_return_val_if_fail(name != NULL, NULL);
+	cm_return_val_if_fail(folder != NULL, NULL);
+	cm_return_val_if_fail(parent != NULL, NULL);
+	cm_return_val_if_fail(name != NULL, NULL);
 
 	path = folder_item_get_path(parent);
 	if (!is_dir_exist(path)) 
@@ -895,10 +928,10 @@ static gint mh_rename_folder(Folder *folder, FolderItem *item,
 	gchar *newpath, *utf8newpath;
 	gchar *paths[2];
 
-	g_return_val_if_fail(folder != NULL, -1);
-	g_return_val_if_fail(item != NULL, -1);
-	g_return_val_if_fail(item->path != NULL, -1);
-	g_return_val_if_fail(name != NULL, -1);
+	cm_return_val_if_fail(folder != NULL, -1);
+	cm_return_val_if_fail(item != NULL, -1);
+	cm_return_val_if_fail(item->path != NULL, -1);
+	cm_return_val_if_fail(name != NULL, -1);
 
 	oldpath = folder_item_get_path(item);
 	if (!is_dir_exist(oldpath))
@@ -944,9 +977,9 @@ static gint mh_remove_folder(Folder *folder, FolderItem *item)
 {
 	gchar *path;
 
-	g_return_val_if_fail(folder != NULL, -1);
-	g_return_val_if_fail(item != NULL, -1);
-	g_return_val_if_fail(item->path != NULL, -1);
+	cm_return_val_if_fail(folder != NULL, -1);
+	cm_return_val_if_fail(item != NULL, -1);
+	cm_return_val_if_fail(item->path != NULL, -1);
 
 	path = folder_item_get_path(item);
 	if (remove_dir_recursive(path) < 0) {
@@ -965,8 +998,8 @@ static MsgInfo *mh_parse_msg(const gchar *file, FolderItem *item)
 	MsgInfo *msginfo;
 	MsgFlags flags;
 
-	g_return_val_if_fail(item != NULL, NULL);
-	g_return_val_if_fail(file != NULL, NULL);
+	cm_return_val_if_fail(item != NULL, NULL);
+	cm_return_val_if_fail(file != NULL, NULL);
 
 	flags.perm_flags = MSG_NEW|MSG_UNREAD;
 	flags.tmp_flags = 0;
@@ -991,7 +1024,7 @@ static gboolean mh_remove_missing_folder_items_func(GNode *node, gpointer data)
 	FolderItem *item;
 	gchar *path;
 
-	g_return_val_if_fail(node->data != NULL, FALSE);
+	cm_return_val_if_fail(node->data != NULL, FALSE);
 
 	if (G_NODE_IS_ROOT(node))
 		return FALSE;
@@ -1010,7 +1043,7 @@ static gboolean mh_remove_missing_folder_items_func(GNode *node, gpointer data)
 
 static void mh_remove_missing_folder_items(Folder *folder)
 {
-	g_return_if_fail(folder != NULL);
+	cm_return_if_fail(folder != NULL);
 
 	debug_print("searching missing folders...\n");
 
@@ -1032,8 +1065,8 @@ static void mh_scan_tree_recursive(FolderItem *item)
  	gchar *real_path, *entry, *utf8entry, *utf8name;
 	gint n_msg = 0;
 
-	g_return_if_fail(item != NULL);
-	g_return_if_fail(item->folder != NULL);
+	cm_return_if_fail(item != NULL);
+	cm_return_if_fail(item->folder != NULL);
 
 	folder = item->folder;
 
@@ -1077,12 +1110,12 @@ static void mh_scan_tree_recursive(FolderItem *item)
 		entry = mh_filename_from_utf8(utf8entry);
 
 		if (
-#if !defined(G_OS_WIN32) && !defined(MAEMO) && defined(HAVE_DIRENT_D_TYPE)
+#if !defined(G_OS_WIN32) && defined(HAVE_DIRENT_D_TYPE)
 			d->d_type == DT_DIR ||
 			(d->d_type == DT_UNKNOWN &&
 #endif
 			g_stat(entry, &s) == 0 && S_ISDIR(s.st_mode)
-#if !defined(G_OS_WIN32) && !defined(MAEMO) && defined(HAVE_DIRENT_D_TYPE)
+#if !defined(G_OS_WIN32) && defined(HAVE_DIRENT_D_TYPE)
 			)
 #endif
 		   ) {
@@ -1239,104 +1272,6 @@ static gchar *get_unseen_seq_name(void)
 	return seq_name;	
 }
 
-#if 0
-static gint mh_get_flags(Folder *folder, FolderItem *item,
-                           MsgInfoList *msginfo_list, GRelation *msgflags)
-{
-	gchar *mh_sequences_filename;
-	FILE *mh_sequences_file;
-	gchar buf[BUFFSIZE];
-	gchar *unseen_list = NULL;
-	gchar *path;
-	MsgInfoList *mcur = NULL;
-/*
-	GTimer *timer = g_timer_new();
-	g_timer_start(timer);
-*/
-	if (!item)
-		return 0;
-
-	/* don't update from .mh_sequences if the item's opened: mails may have
-	 * been marked read/unread and it's not yet written in the file. */	
-	if (item->opened)
-		return 0;
-
-	path = folder_item_get_path(item);
-
-	mh_sequences_filename = g_strconcat(path, G_DIR_SEPARATOR_S,
-					    ".mh_sequences", NULL);
-	g_free(path);
-	if ((mh_sequences_file = g_fopen(mh_sequences_filename, "r+b")) != NULL) {
-		while (fgets(buf, sizeof(buf), mh_sequences_file) != NULL) {
-			if (!strncmp(buf, get_unseen_seq_name(), strlen(get_unseen_seq_name()))) {
-				unseen_list = g_strdup(buf+strlen(get_unseen_seq_name()));
-				break;
-			}
-		}
-		fclose(mh_sequences_file);
-	}
-	
-	g_free(mh_sequences_filename);
-	
-	if (unseen_list) {
-		gchar *cur = NULL;
-		gchar *token = NULL, *next = NULL, *boundary = NULL;
-		gint num = 0;
-		GHashTable *unseen_table = g_hash_table_new(g_direct_hash, g_direct_equal);
-
-		cur = unseen_list = strretchomp(unseen_list);
-		debug_print("found unseen list in .mh_sequences: %s\n", unseen_list);
-next_token:
-		while (*cur && *cur == ' ')
-			cur++;
-		
-		if ((next = strchr(cur, ' ')) != NULL) {
-			token = cur;
-			cur = next+1;
-			*next = '\0';
-		} else {
-			token = cur;
-			cur = NULL;
-		}
-		
-		if ((boundary = strchr(token, '-')) != NULL) {
-			gchar *start, *end;
-			int i;
-			start = token;
-			end = boundary+1;
-			*boundary='\0';
-			for (i = atoi(start); i <= atoi(end); i++) {
-				g_hash_table_insert(unseen_table, GINT_TO_POINTER(i), GINT_TO_POINTER(1));
-			}
-		} else if ((num = atoi(token)) > 0) {
-			g_hash_table_insert(unseen_table, GINT_TO_POINTER(num), GINT_TO_POINTER(1));
-		}
-		
-		if (cur)
-			goto next_token;
-		for (mcur = msginfo_list; mcur; mcur = mcur->next) {
-			MsgInfo *msginfo = (MsgInfo *)mcur->data;
-			MsgPermFlags flags = msginfo->flags.perm_flags;
-			if (g_hash_table_lookup(unseen_table, GINT_TO_POINTER(msginfo->msgnum))) {
-				flags |= MSG_UNREAD;
-			} else if (!(flags & MSG_NEW)) { /* don't mark new msgs as read */
-				flags &= ~(MSG_UNREAD);
-			}
-			if (flags != msginfo->flags.perm_flags)
-				g_relation_insert(msgflags, msginfo, GINT_TO_POINTER(flags));
-		}
-		g_hash_table_destroy(unseen_table);
-		g_free(unseen_list);
-	}
-/*
-	g_timer_stop(timer);
-	g_print("mh_get_flags: %f secs\n", g_timer_elapsed(timer, NULL));
-	g_timer_destroy(timer);
-*/
-	return 0;
-}
-#endif
-
 static void mh_write_sequences(FolderItem *item, gboolean remove_unseen)
 {
 	gchar *mh_sequences_old, *mh_sequences_new;
@@ -1451,7 +1386,7 @@ static void mh_set_mtime(Folder *folder, FolderItem *item)
 	struct stat s;
 	gchar *path = folder_item_get_path(item);
 
-	g_return_if_fail(path != NULL);
+	cm_return_if_fail(path != NULL);
 
 	if (g_stat(path, &s) < 0) {
 		FILE_OP_ERROR(path, "stat");
