@@ -198,7 +198,7 @@ static void bogofilter_do_filter(BogoFilterData *data)
 	int status = 0;
 	MsgInfo *msginfo;
 	GSList *cur = NULL;
-	int total = 0, curnum = 0;
+	int total = 0, curnum = 1;
 	gchar *file = NULL;
 	gchar buf[BUFSIZ];
 
@@ -290,7 +290,7 @@ static void bogofilter_do_filter(BogoFilterData *data)
 							const gchar *bogosity = *parts[1] == 'S' ? "Spam":
 										 (*parts[1] == 'H' ? "Ham":"Unsure");
 							gchar *tmpstr = g_strdup_printf(
-									"X-Claws-Bogosity: %s, spamicity=%s%s\n",
+									"X-Bogosity: %s, spamicity=%s%s\n",
 									bogosity, parts[2],
 									whitelisted?" [whitelisted]":"");
 							if (fwrite(tmpstr, 1, strlen(tmpstr), output) < strlen(tmpstr)) {
