@@ -27,8 +27,11 @@
 
 #include "nodetype.h"
 #include "composewindowtype.h"
+#include "folderpropertiestype.h"
 #include "foldertype.h"
 #include "messageinfotype.h"
+#include "accounttype.h"
+#include "mailboxtype.h"
 
 #include <pygobject.h>
 #include <pygtk/pygtk.h>
@@ -40,6 +43,7 @@
 #include "toolbar.h"
 #include "prefs_common.h"
 #include "common/tags.h"
+#include "account.h"
 
 #include <glib.h>
 
@@ -103,29 +107,70 @@ static PyObject *get_folderview_selected_folder(PyObject *self, PyObject *args)
     if(item)
       return clawsmail_folder_new(item);
   }
-  Py_INCREF(Py_None);
-  return Py_None;
+  Py_RETURN_NONE;
 }
 
-static PyObject *folderview_select_folder(PyObject *self, PyObject *args)
+static PyObject *get_folderview_selected_mailbox(PyObject *self, PyObject *args)
 {
   MainWindow *mainwin;
 
   mainwin =  mainwindow_get_mainwindow();
   if(mainwin && mainwin->folderview) {
     FolderItem *item;
-    PyObject *folder;
-    folder = PyTuple_GetItem(args, 0);
-    if(!folder)
-      return NULL;
-    Py_INCREF(folder);
-    item = clawsmail_folder_get_item(folder);
-    Py_DECREF(folder);
-    if(item)
-      folderview_select(mainwin->folderview, item);
+    item = folderview_get_selected_item(mainwin->folderview);
+    if(item) {
+      gchar *id;
+      id = folder_item_get_identifier(item);
+      /* If there is an id, it's a folder, not a mailbox */
+      if(id) {
+        g_free(id);
+        Py_RETURN_NONE;
+      }
+      else
+        return clawsmail_mailbox_new(item->folder);
+    }
   }
-  Py_INCREF(Py_None);
-  return Py_None;
+  Py_RETURN_NONE;
+}
+
+static PyObject *folderview_select_row(PyObject *self, PyObject *args)
+{
+  MainWindow *mainwin;
+  gboolean ok;
+
+  ok = TRUE;
+  mainwin =  mainwindow_get_mainwindow();
+  if(mainwin && mainwin->folderview) {
+    PyObject *arg;
+    arg = PyTuple_GetItem(args, 0);
+    if(!arg)
+      return NULL;
+    Py_INCREF(arg);
+
+    if(clawsmail_folder_check(arg)) {
+      FolderItem *item;
+      item = clawsmail_folder_get_item(arg);
+      if(item)
+        folderview_select(mainwin->folderview, item);
+    }
+    else if(clawsmail_mailbox_check(arg)) {
+      Folder *folder;
+      folder = clawsmail_mailbox_get_folder(arg);
+      if(folder && folder->node) {
+        folderview_select(mainwin->folderview, folder->node->data);
+      }
+    }
+    else {
+      PyErr_SetString(PyExc_TypeError, "Bad argument type");
+      ok = FALSE;
+    }
+
+    Py_DECREF(arg);
+  }
+  if(ok)
+    Py_RETURN_NONE;
+  else
+    return NULL;
 }
 
 static gboolean setup_folderitem_node(GNode *item_node, GNode *item_parent, PyObject **pyparent)
@@ -177,6 +222,29 @@ static gboolean setup_folderitem_node(GNode *item_node, GNode *item_parent, PyOb
   return TRUE;
 }
 
+static PyObject* get_folder_tree_from_folder(Folder *folder)
+{
+  if(folder->node) {
+    PyObject *root;
+    int n_children, i_child;
+
+    /* create root nodes */
+    root = clawsmail_node_new(cm_module);
+    if(!root)
+      return NULL;
+
+    n_children = g_node_n_children(folder->node);
+    for(i_child = 0; i_child < n_children; i_child++) {
+      if(!setup_folderitem_node(g_node_nth_child(folder->node, i_child), folder->node, &root)) {
+        Py_DECREF(root);
+        return NULL;
+      }
+    }
+    return root;
+  }
+  return NULL;
+}
+
 static PyObject* get_folder_tree_from_account_name(const char *str)
 {
   PyObject *result;
@@ -188,28 +256,19 @@ static PyObject* get_folder_tree_from_account_name(const char *str)
 
   for(walk = folder_get_list(); walk; walk = walk->next) {
     Folder *folder = walk->data;
-    if((!str || !g_strcmp0(str, folder->name)) && folder->node) {
-      PyObject *root;
-      int n_children, i_child, retval;
-
-      /* create root nodes */
-      root = clawsmail_node_new(cm_module);
-      if(!root) {
-        Py_DECREF(result);
-        return NULL;
-      }
-
-      n_children = g_node_n_children(folder->node);
-      for(i_child = 0; i_child < n_children; i_child++) {
-        if(!setup_folderitem_node(g_node_nth_child(folder->node, i_child), folder->node, &root)) {
-          Py_DECREF(root);
+    if(!str || !g_strcmp0(str, folder->name)) {
+      PyObject *tree_from_folder;
+      tree_from_folder = get_folder_tree_from_folder(folder);
+      if(tree_from_folder) {
+        int retval;
+        retval = PyList_Append(result, tree_from_folder);
+        Py_DECREF(tree_from_folder);
+        if(retval == -1) {
           Py_DECREF(result);
           return NULL;
         }
       }
-      retval = PyList_Append(result, root);
-      Py_DECREF(root);
-      if(retval == -1) {
+      else {
         Py_DECREF(result);
         return NULL;
       }
@@ -275,8 +334,11 @@ static PyObject* get_folder_tree(PyObject *self, PyObject *args)
   else if(PyObject_TypeCheck(arg, clawsmail_folder_get_type_object())) {
     result = get_folder_tree_from_folderitem(clawsmail_folder_get_item(arg));
   }
+  else if(clawsmail_mailbox_check(arg)) {
+    result = get_folder_tree_from_folder(clawsmail_mailbox_get_folder(arg));
+  }
   else {
-    PyErr_SetString(PyExc_TypeError, "Parameter must be nothing, a mailbox string or a Folder object.");
+    PyErr_SetString(PyExc_TypeError, "Parameter must be nothing, a Folder object, a Mailbox object, or a mailbox name string.");
     return NULL;
   }
 
@@ -434,6 +496,61 @@ static PyObject* get_tags(PyObject *self, PyObject *args)
   return tags_tuple;
 }
 
+static PyObject* get_accounts(PyObject *self, PyObject *args)
+{
+  PyObject *accounts_tuple;
+  GList *accounts_list;
+  GList *walk;
+
+  accounts_list = account_get_list();
+
+  accounts_tuple = PyTuple_New(g_list_length(accounts_list));
+  if(accounts_tuple) {
+    PyObject *account_object;
+    Py_ssize_t iAccount;
+
+    iAccount = 0;
+    for(walk = accounts_list; walk; walk = walk->next) {
+      account_object = clawsmail_account_new(walk->data);
+      if(account_object == NULL) {
+        Py_DECREF(accounts_tuple);
+        return NULL;
+      }
+      PyTuple_SET_ITEM(accounts_tuple, iAccount++, account_object);
+    }
+  }
+
+  return accounts_tuple;
+}
+
+static PyObject* get_mailboxes(PyObject *self, PyObject *args)
+{
+  PyObject *mailboxes_tuple;
+  GList *mailboxes_list;
+  GList *walk;
+
+  mailboxes_list = folder_get_list();
+
+  mailboxes_tuple = PyTuple_New(g_list_length(mailboxes_list));
+  if(mailboxes_tuple) {
+    PyObject *mailbox_object;
+    Py_ssize_t iMailbox;
+
+    iMailbox = 0;
+    for(walk = mailboxes_list; walk; walk = walk->next) {
+      mailbox_object = clawsmail_mailbox_new(walk->data);
+      if(mailbox_object == NULL) {
+        Py_DECREF(mailboxes_tuple);
+        return NULL;
+      }
+      PyTuple_SET_ITEM(mailboxes_tuple, iMailbox++, mailbox_object);
+    }
+  }
+
+  return mailboxes_tuple;
+}
+
+
 static PyObject* make_sure_tag_exists(PyObject *self, PyObject *args)
 {
   int retval;
@@ -519,7 +636,7 @@ static gboolean get_message_list_for_move_or_copy(PyObject *messagelist, PyObjec
 {
   Py_ssize_t size, iEl;
   FolderItem *folderitem;
-  
+
   *list = NULL;
 
   folderitem = clawsmail_folder_get_item(folder);
@@ -537,17 +654,17 @@ static gboolean get_message_list_for_move_or_copy(PyObject *messagelist, PyObjec
       PyErr_SetString(PyExc_TypeError, "Argument must be a list of MessageInfo objects.");
       return FALSE;
     }
-    
+
     msginfo = clawsmail_messageinfo_get_msginfo(element);
     if(!msginfo) {
       PyErr_SetString(PyExc_LookupError, "Broken MessageInfo object.");
       return FALSE;
     }
-   
+
     procmsg_msginfo_set_to_folder(msginfo, folderitem);
     *list = g_slist_prepend(*list, msginfo);
   }
- 
+
   return TRUE;
 }
 
@@ -557,23 +674,23 @@ static PyObject* move_or_copy_messages(PyObject *self, PyObject *args, gboolean
   PyObject *folder;
   int retval;
   GSList *list = NULL;
-  
+
   retval = PyArg_ParseTuple(args, "O!O!",
     &PyList_Type, &messagelist,
     clawsmail_folder_get_type_object(), &folder);
   if(!retval )
-    return NULL;  
+    return NULL;
 
   folder_item_update_freeze();
-  
+
   if(!get_message_list_for_move_or_copy(messagelist, folder, &list))
     goto err;
-  
-  if(move)   
+
+  if(move)
     procmsg_move_messages(list);
   else
     procmsg_copy_messages(list);
-      
+
   folder_item_update_thaw();
   g_slist_free(list);
   Py_RETURN_NONE;
@@ -595,6 +712,29 @@ static PyObject* copy_messages(PyObject *self, PyObject *args)
   return move_or_copy_messages(self, args, FALSE);
 }
 
+static PyObject* get_current_account(PyObject *self, PyObject *args)
+{
+  PrefsAccount *account;
+  account = account_get_cur_account();
+  if(account) {
+    return clawsmail_account_new(account);
+  }
+  else
+    Py_RETURN_NONE;
+}
+
+static PyObject* get_default_account(PyObject *self, PyObject *args)
+{
+  PrefsAccount *account;
+  account = account_get_default();
+  if(account) {
+    return clawsmail_account_new(account);
+  }
+  else
+    Py_RETURN_NONE;
+}
+
+
 static PyMethodDef ClawsMailMethods[] = {
     /* public */
     {"get_mainwindow_action_group",  get_mainwindow_action_group, METH_NOARGS,
@@ -602,7 +742,7 @@ static PyMethodDef ClawsMailMethods[] = {
      "\n"
      "Returns the gtk.ActionGroup for the main window."},
 
-    {"get_mainwindow_ui_manager",  get_mainwindow_ui_manager, METH_NOARGS, 
+    {"get_mainwindow_ui_manager",  get_mainwindow_ui_manager, METH_NOARGS,
      "get_mainwindow_ui_manager() - get ui manager of main window\n"
      "\n"
      "Returns the gtk.UIManager for the main window."},
@@ -612,22 +752,38 @@ static PyMethodDef ClawsMailMethods[] = {
      "\n"
      "Without arguments, get a list of folder trees for all mailboxes.\n"
      "\n"
-     "If the optional root argument is a string, it is supposed to be a\n"
-     "mailbox name. The function then returns a tree of folders of that mailbox.\n"
-     "\n"
      "If the optional root argument is a clawsmail.Folder, the function\n"
      "returns a tree of subfolders with the given folder as root element.\n"
      "\n"
+     "If the optional root argument is a clawsmail.Mailbox, the function\n"
+     "returns a tree of folders with the given mailbox as root element.\n"
+     "\n"
+     "If the optional root argument is a string, it is supposed to be a\n"
+     "mailbox name. The function then returns a tree of folders of that mailbox.\n"
+     "\n"
      "In any case, a tree consists of elements of the type clawsmail.Node."},
 
     {"get_folderview_selected_folder",  get_folderview_selected_folder, METH_NOARGS,
      "get_folderview_selected_folder() - get selected folder in folderview\n"
      "\n"
-     "Returns the currently selected folder as a clawsmail.Folder."},
-    {"folderview_select_folder",  folderview_select_folder, METH_VARARGS,
+     "Returns the currently selected folder as a clawsmail.Folder or None if no folder is selected."},
+    {"folderview_select_folder",  folderview_select_row, METH_VARARGS,
      "folderview_select_folder(folder) - select folder in folderview\n"
      "\n"
-     "Takes an argument of type clawsmail.Folder, and selects the corresponding folder."},
+     "Takes an argument of type clawsmail.Folder, and selects the corresponding folder.\n"
+     "\n"
+     "DEPRECATED: Use folderview_select() instead."},
+
+    {"get_folderview_selected_mailbox",  get_folderview_selected_mailbox, METH_NOARGS,
+     "get_folderview_selected_mailbox() - get selected mailbox in folderview\n"
+     "\n"
+     "Returns the currently selected mailbox as a clawsmail.Mailbox or None if no mailbox is selected."},
+
+     {"folderview_select",  folderview_select_row, METH_VARARGS,
+      "folderview_select(arg) - select folder or a mailbox in folderview\n"
+      "\n"
+      "Takes an argument of type clawsmail.Folder or clawsmail.Mailbox, and selects the corresponding\n"
+      "row in the folder view."},
 
     {"quicksearch_search", quicksearch_search, METH_VARARGS,
      "quicksearch_search(string [, type]) - perform a quicksearch\n"
@@ -696,6 +852,26 @@ static PyMethodDef ClawsMailMethods[] = {
      "Raises a KeyError exception if the tag does not exist.\n"
      "Raises a ValueError exception if the old or new tag name is a reserved name."},
 
+     {"get_accounts", get_accounts, METH_NOARGS,
+      "get_accounts() - get a tuple of all accounts that Claws Mail knows about\n"
+      "\n"
+      "Get a tuple of Account objects representing all accounts that are defined in Claws Mail."},
+
+      {"get_current_account", get_current_account, METH_NOARGS,
+       "get_current_account() - get the current account\n"
+       "\n"
+       "Return the object representing the currently selected account."},
+
+     {"get_default_account", get_default_account, METH_NOARGS,
+      "get_default_account() - get the default account\n"
+      "\n"
+      "Return the object representing the default account."},
+
+     {"get_mailboxes", get_mailboxes, METH_NOARGS,
+      "get_mailboxes() - get a tuple of all mailboxes that Claws Mail knows about\n"
+      "\n"
+      "Get a tuple of Mailbox objects representing all mailboxes that are defined in Claws Mail."},
+
      /* private */
     {"__gobj", private_wrap_gobj, METH_VARARGS,
      "__gobj(ptr) - transforms a C GObject pointer into a PyGObject\n"
@@ -705,8 +881,9 @@ static PyMethodDef ClawsMailMethods[] = {
     {NULL, NULL, 0, NULL}
 };
 
-static void initmiscstuff(PyObject *module)
+static gboolean add_miscstuff(PyObject *module)
 {
+  gboolean retval;
   PyObject *dict;
   PyObject *res;
   const char *cmd =
@@ -719,14 +896,15 @@ static void initmiscstuff(PyObject *module)
       "\n";
   dict = PyModule_GetDict(module);
   res = PyRun_String(cmd, Py_file_input, dict, dict);
+  retval = (res != NULL);
   Py_XDECREF(res);
+  return retval;
 }
 
 
-void claws_mail_python_init(void)
+PyMODINIT_FUNC initclawsmail(void)
 {
-  if (!Py_IsInitialized())
-      Py_Initialize();
+  gboolean ok = TRUE;
 
   /* create module */
   cm_module = Py_InitModule3("clawsmail", ClawsMailMethods,
@@ -740,17 +918,22 @@ void claws_mail_python_init(void)
       "The interface to Claws Mail in this module is extended on a 'as-needed' basis.\n"
       "If you're missing something specific, try contacting the author.");
 
+  /* add module member "compose_window" set to None */
+  Py_INCREF(Py_None);
+  PyModule_AddObject(cm_module, "compose_window", Py_None);
+
   /* initialize classes */
-  initnode(cm_module);
-  initcomposewindow(cm_module);
-  initfolder(cm_module);
-  initmessageinfo(cm_module);
+  ok = ok && cmpy_add_node(cm_module);
+  ok = ok && cmpy_add_composewindow(cm_module);
+  ok = ok && cmpy_add_folder(cm_module);
+  ok = ok && cmpy_add_messageinfo(cm_module);
+  ok = ok && cmpy_add_account(cm_module);
+  ok = ok && cmpy_add_folderproperties(cm_module);
+  ok = ok && cmpy_add_mailbox(cm_module);
 
   /* initialize misc things */
-  initmiscstuff(cm_module);
-
-  PyRun_SimpleString("import clawsmail\n");
-  PyRun_SimpleString("clawsmail.compose_window = None\n");
+  if(ok)
+    add_miscstuff(cm_module);
 }
 
 
