@@ -130,7 +130,8 @@ static void set_auth_sensitivity(struct VcalendarPage *page)
 	     !strncmp(export_path, "ftp://", 6) ||
 	     !strncmp(export_path, "https://", 8) ||
 	     !strncmp(export_path, "sftp://", 5) ||
-	     !strncmp(export_path, "webcal://", 9))) {
+	     !strncmp(export_path, "webcal://", 9) ||
+	     !strncmp(export_path, "webcals://", 10))) {
 		gtk_widget_set_sensitive(page->export_user_label, TRUE);	
 		gtk_widget_set_sensitive(page->export_user_entry, TRUE);	
 		gtk_widget_set_sensitive(page->export_pass_label, TRUE);	
@@ -146,7 +147,8 @@ static void set_auth_sensitivity(struct VcalendarPage *page)
 	     !strncmp(export_freebusy_path, "ftp://", 6) ||
 	     !strncmp(export_freebusy_path, "https://", 8) ||
 	     !strncmp(export_freebusy_path, "sftp://", 5) ||
-	     !strncmp(export_freebusy_path, "webcal://", 9))) {
+	     !strncmp(export_freebusy_path, "webcal://", 9) ||
+	     !strncmp(export_freebusy_path, "webcals://", 10))) {
 		gtk_widget_set_sensitive(page->export_freebusy_user_label, TRUE);	
 		gtk_widget_set_sensitive(page->export_freebusy_user_entry, TRUE);	
 		gtk_widget_set_sensitive(page->export_freebusy_pass_label, TRUE);	
