@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -64,6 +65,7 @@ static struct CustomHdr {
 	GtkWidget *hdr_combo;
 	GtkWidget *hdr_entry;
 	GtkWidget *val_entry;
+	GtkWidget *preview;
 	GtkWidget *list_view;
 } customhdr;
 
@@ -124,6 +126,7 @@ void prefs_custom_header_open(PrefsAccount *ac)
 	cur_ac = ac;
 
 	gtk_widget_show(customhdr.window);
+	gtk_window_set_modal(GTK_WINDOW(customhdr.window), TRUE);
 }
 
 static void prefs_custom_header_create(void)
@@ -150,6 +153,7 @@ static void prefs_custom_header_create(void)
 	GtkWidget *arrow;
 	GtkWidget *add_btn;
 	GtkWidget *del_btn;
+	GtkWidget *preview;
 
 	GtkWidget *ch_hbox;
 	GtkWidget *ch_scrolledwin;
@@ -164,7 +168,6 @@ static void prefs_custom_header_create(void)
 	window = gtkut_window_new(GTK_WINDOW_TOPLEVEL, "prefs_customheader");
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
 	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
-	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
 	gtk_window_set_resizable(GTK_WINDOW (window), TRUE);
 
 	vbox = gtk_vbox_new (FALSE, 6);
@@ -231,7 +234,7 @@ static void prefs_custom_header_create(void)
 			  0, 0, 0);
 	gtk_widget_set_size_request (val_entry, 200, -1);
 
-	val_btn = gtk_button_new_with_label (_("From file..."));
+	val_btn = gtkut_get_browse_file_btn(_("Bro_wse"));
 	gtk_widget_show (val_btn);
 	gtk_table_attach (GTK_TABLE (table1), val_btn, 2, 3, 1, 2,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
@@ -302,11 +305,16 @@ static void prefs_custom_header_create(void)
 	g_signal_connect (G_OBJECT (down_btn), "clicked",
 			  G_CALLBACK (prefs_custom_header_down), NULL);
 
+	preview = gtk_image_new ();
+	gtk_widget_show (preview);
+	gtk_box_pack_start (GTK_BOX (btn_vbox), preview, FALSE, FALSE, 0);
+
 	gtk_widget_show_all(window);
 
 	customhdr.window     = window;
 	customhdr.ok_btn     = ok_btn;
 	customhdr.cancel_btn = cancel_btn;
+	customhdr.preview = preview;
 
 	customhdr.hdr_combo  = hdr_combo;
 	customhdr.hdr_entry  = gtk_bin_get_child(GTK_BIN((hdr_combo)));
@@ -493,10 +501,6 @@ static void prefs_custom_header_list_view_set_row(PrefsAccount *ac)
 	CustomHeader *ch;
 	const gchar *entry_text;
 	gchar *ch_str;
-	GtkListStore *store;
-
-	store = GTK_LIST_STORE(gtk_tree_view_get_model
-				(GTK_TREE_VIEW(customhdr.list_view)));
 
 	entry_text = gtk_entry_get_text(GTK_ENTRY(customhdr.hdr_entry));
 	if (entry_text[0] == '\0') {
@@ -771,7 +775,7 @@ static gboolean prefs_custom_header_key_pressed(GtkWidget *widget,
 						GdkEventKey *event,
 						gpointer data)
 {
-	if (event && event->keyval == GDK_Escape)
+	if (event && event->keyval == GDK_KEY_Escape)
 		prefs_custom_header_cancel();
 	return FALSE;
 }
@@ -780,12 +784,14 @@ static void prefs_custom_header_ok(void)
 {
 	prefs_custom_header_write_config(cur_ac);
 	gtk_widget_hide(customhdr.window);
+	gtk_window_set_modal(GTK_WINDOW(customhdr.window), FALSE);
 }
 
 static void prefs_custom_header_cancel(void)
 {
 	prefs_custom_header_read_config(cur_ac); 
 	gtk_widget_hide(customhdr.window);
+	gtk_window_set_modal(GTK_WINDOW(customhdr.window), FALSE);
 }
 
 static gint prefs_custom_header_deleted(GtkWidget *widget, GdkEventAny *event,
@@ -847,6 +853,7 @@ static GtkWidget *prefs_custom_header_list_view_create(void)
 	g_object_unref(model);	
 	
 	gtk_tree_view_set_rules_hint(list_view, prefs_common.use_stripes_everywhere);
+	gtk_tree_view_set_reorderable(list_view, TRUE);
 	
 	selector = gtk_tree_view_get_selection(list_view);
 	gtk_tree_selection_set_mode(selector, GTK_SELECTION_BROWSE);
@@ -884,6 +891,8 @@ static gboolean prefs_custom_header_selected(GtkTreeSelection *selector,
 {
 	GtkTreeIter iter;
 	CustomHeader *ch;
+	GtkImage *preview;
+	GdkPixbuf *pixbuf;
 	CustomHeader default_ch = { 0, "", NULL };
 
 	if (currently_selected)
@@ -900,7 +909,39 @@ static gboolean prefs_custom_header_selected(GtkTreeSelection *selector,
 
 	ENTRY_SET_TEXT(customhdr.hdr_entry, ch->name);
 	ENTRY_SET_TEXT(customhdr.val_entry, ch->value);
-			   
+	if (!strcmp2("Face",ch->name)) {
+		preview = GTK_IMAGE(face_get_from_header (ch->value));
+		pixbuf = gtk_image_get_pixbuf(preview);
+		gtk_image_set_from_pixbuf (GTK_IMAGE(customhdr.preview), pixbuf);
+		gtk_widget_show(customhdr.preview);
+#if GLIB_CHECK_VERSION(2,10,0)
+		g_object_ref_sink (G_OBJECT(preview));
+#else
+		gtk_object_ref (G_OBJECT(preview));
+		gtk_object_sink (G_OBJECT(preview));
+#endif
+	} 
+#if HAVE_LIBCOMPFACE
+else if (!strcmp2("X-Face", ch->name)) {
+		GdkColor color;
+		color.pixel = 0;
+		preview = GTK_IMAGE(xface_get_from_header(ch->value, 
+					  &color, 
+					  mainwindow_get_mainwindow()->window->window));	
+		pixbuf = gtk_image_get_pixbuf(preview);
+		gtk_image_set_from_pixbuf (GTK_IMAGE(customhdr.preview), pixbuf);
+		gtk_widget_show(customhdr.preview);
+#if GLIB_CHECK_VERSION(2,10,0)
+		g_object_ref_sink (G_OBJECT(preview));
+#else
+		gtk_object_ref (G_OBJECT(preview));
+		gtk_object_sink (G_OBJECT(preview));
+#endif
+	} 
+#endif
+else {
+		gtk_widget_hide(customhdr.preview);
+	}
 	return TRUE;
 }
 
