@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -55,7 +56,13 @@
 #include "html.h"
 #include "enriched.h"
 #include "compose.h"
-#include "addressbook.h"
+#ifndef USE_NEW_ADDRBOOK
+	#include "addressbook.h"
+	#include "addrindex.h"
+#else
+	#include "addressbook-dbus.h"
+	#include "addressadd.h"
+#endif
 #include "displayheader.h"
 #include "account.h"
 #include "mimeview.h"
@@ -67,7 +74,8 @@
 #include "inputdialog.h"
 #include "timing.h"
 #include "tags.h"
-#include "addrindex.h"
+#include "manage_window.h"
+#include "folder_item_prefs.h"
 
 static GdkColor quote_colors[3] = {
 	{(gulong)0, (gushort)0, (gushort)0, (gushort)0},
@@ -101,6 +109,27 @@ static GdkColor emphasis_color = {
 	(gushort)0
 };
 
+static GdkColor diff_added_color = {
+	(gulong)0,
+	(gushort)0,
+	(gushort)0,
+	(gushort)0
+};
+
+static GdkColor diff_deleted_color = {
+	(gulong)0,
+	(gushort)0,
+	(gushort)0,
+	(gushort)0
+};
+
+static GdkColor diff_hunk_color = {
+	(gulong)0,
+	(gushort)0,
+	(gushort)0,
+	(gushort)0
+};
+
 static GdkCursor *hand_cursor = NULL;
 static GdkCursor *text_cursor = NULL;
 static GdkCursor *watch_cursor= NULL;
@@ -233,8 +262,10 @@ static void scrolled_cb (GtkAdjustment *adj, TextView *textview)
 #  define HEIGHT 48
 #endif
 	if (textview->image) {
+		GtkAllocation allocation;
 		gint x, y, x1;
-		x1 = textview->text->allocation.width - WIDTH - 5;
+		gtk_widget_get_allocation(textview->text, &allocation);
+		x1 = allocation.width - WIDTH - 5;
 		gtk_text_view_buffer_to_window_coords(
 			GTK_TEXT_VIEW(textview->text),
 			GTK_TEXT_WINDOW_TEXT, x1, 5, &x, &y);
@@ -368,7 +399,6 @@ TextView *textview_create(void)
 	textview->text               = text;
 	textview->uri_list           = NULL;
 	textview->body_pos           = 0;
-	textview->show_all_headers   = FALSE;
 	textview->last_buttonpress   = GDK_NOTHING;
 	textview->image		     = NULL;
 	return textview;
@@ -378,10 +408,16 @@ static void textview_create_tags(GtkTextView *text, TextView *textview)
 {
 	GtkTextBuffer *buffer;
 	GtkTextTag *tag, *qtag;
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	static GdkColor yellow, black;
 	static gboolean color_init = FALSE;
+#else
+	static GdkColor yellow = { (guint32)0, (guint16)0xf5, (guint16)0xf6, (guint16)0xbe };
+	static GdkColor black = { (guint32)0, (guint16)0x0, (guint16)0x0, (guint16)0x0 };
+#endif
 	static PangoFontDescription *font_desc, *bold_font_desc;
 	
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	if (!color_init) {
 		gdk_color_parse("#f5f6be", &yellow);
 		gdk_color_parse("#000000", &black);
@@ -390,6 +426,7 @@ static void textview_create_tags(GtkTextView *text, TextView *textview)
 		color_init &= gdk_colormap_alloc_color(
 			gdk_colormap_get_system(), &black, FALSE, TRUE);
 	}
+#endif
 
 	if (!font_desc)
 		font_desc = pango_font_description_from_string
@@ -475,6 +512,24 @@ static void textview_create_tags(GtkTextView *text, TextView *textview)
 	gtk_text_buffer_create_tag(buffer, "link-hover",
 			"underline", PANGO_UNDERLINE_SINGLE,
 			NULL);
+	gtk_text_buffer_create_tag(buffer, "diff-add",
+			"foreground-gdk", &diff_added_color,
+			NULL);
+	gtk_text_buffer_create_tag(buffer, "diff-del",
+			"foreground-gdk", &diff_deleted_color,
+			NULL);
+	gtk_text_buffer_create_tag(buffer, "diff-add-file",
+			"foreground-gdk", &diff_added_color,
+			"weight", PANGO_WEIGHT_BOLD,
+			NULL);
+	gtk_text_buffer_create_tag(buffer, "diff-del-file",
+			"foreground-gdk", &diff_deleted_color,
+			"weight", PANGO_WEIGHT_BOLD,
+			NULL);
+	gtk_text_buffer_create_tag(buffer, "diff-hunk",
+			"foreground-gdk", &diff_hunk_color,
+			"weight", PANGO_WEIGHT_BOLD,
+			NULL);
 	g_signal_connect(G_OBJECT(qtag), "event",
                          G_CALLBACK(textview_uri_button_pressed), textview);
 	g_signal_connect(G_OBJECT(tag), "event",
@@ -495,7 +550,6 @@ void textview_init(TextView *textview)
 		watch_cursor = gdk_cursor_new(GDK_WATCH);
 
 	textview_reflect_prefs(textview);
-	textview_set_all_headers(textview, FALSE);
 	textview_set_font(textview, NULL);
 	textview_create_tags(GTK_TEXT_VIEW(textview->text), textview);
 }
@@ -515,8 +569,9 @@ static void textview_update_message_colors(TextView *textview)
 	GtkTextTag *tag = NULL;
 
 	quote_bgcolors[0] = quote_bgcolors[1] = quote_bgcolors[2] = black;
-	quote_colors[0] = quote_colors[1] = quote_colors[2] = 
-		uri_color = emphasis_color = signature_color = black;
+	quote_colors[0] = quote_colors[1] = quote_colors[2] = black;
+	uri_color = emphasis_color = signature_color = diff_added_color =
+		diff_deleted_color = diff_hunk_color = black;
 
 	if (prefs_common.enable_color) {
 		/* grab the quote colors, converting from an int to a GdkColor */
@@ -532,6 +587,12 @@ static void textview_update_message_colors(TextView *textview)
 					       &signature_color);
 		gtkut_convert_int_to_gdk_color(prefs_common.emphasis_col,
 					       &emphasis_color);
+		gtkut_convert_int_to_gdk_color(prefs_common.diff_added_color,
+					       &diff_added_color);
+		gtkut_convert_int_to_gdk_color(prefs_common.diff_deleted_color,
+					       &diff_deleted_color);
+		gtkut_convert_int_to_gdk_color(prefs_common.diff_hunk_color,
+					       &diff_hunk_color);
 	}
 	if (prefs_common.enable_color && prefs_common.enable_bgcolor) {
 		gtkut_convert_int_to_gdk_color(prefs_common.quote_level1_bgcol,
@@ -553,6 +614,11 @@ static void textview_update_message_colors(TextView *textview)
 	CHANGE_TAG_COLOR("signature", &signature_color, NULL);
 	CHANGE_TAG_COLOR("link", &uri_color, NULL);
 	CHANGE_TAG_COLOR("link-hover", &uri_color, NULL);
+	CHANGE_TAG_COLOR("diff-add", &diff_added_color, NULL);
+	CHANGE_TAG_COLOR("diff-del", &diff_deleted_color, NULL);
+	CHANGE_TAG_COLOR("diff-add-file", &diff_added_color, NULL);
+	CHANGE_TAG_COLOR("diff-del-file", &diff_deleted_color, NULL);
+	CHANGE_TAG_COLOR("diff-hunk", &diff_hunk_color, NULL);
 }
 #undef CHANGE_TAG_COLOR
 
@@ -603,6 +669,7 @@ void textview_show_part(TextView *textview, MimeInfo *mimeinfo, FILE *fp)
 
 static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
 {
+	GtkAllocation allocation;
 	GtkTextView *text;
 	GtkTextBuffer *buffer;
 	GtkTextIter iter, start_iter;
@@ -679,8 +746,6 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
 		if (mimeinfo->type == MIMETYPE_IMAGE  &&
 		    prefs_common.inline_img ) {
 			GdkPixbuf *pixbuf;
-			gint avail_width;
-			gint avail_height;
 			GError *error = NULL;
 			gchar *filename;
 			ClickableText *uri;
@@ -697,19 +762,9 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
 				return;
 			}
 
-			if (!prefs_common.resize_img) {
-				pixbuf = gdk_pixbuf_new_from_file(filename, &error);
-			} else {
-				gint w, h;
-				gdk_pixbuf_get_file_info(filename, &w, &h);
-				avail_width = textview->scrolledwin->allocation.width;
-				avail_height = textview->scrolledwin->allocation.height;
-				if (avail_width - 100 > 0 &&
-				    (w > avail_width || h > avail_height))
-					pixbuf = gdk_pixbuf_new_from_file_at_scale(filename, 
-						avail_width, avail_height, TRUE, &error);
-				else
-					pixbuf = gdk_pixbuf_new_from_file(filename, &error);
+			pixbuf = gdk_pixbuf_new_from_file(filename, &error);
+			if (textview->stop_loading) {
+				return;
 			}
 			if (error != NULL) {
 				g_warning("%s\n", error->message);
@@ -722,6 +777,15 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
 				return;
 			}
 
+			gtk_widget_get_allocation(textview->scrolledwin, &allocation);
+			pixbuf = claws_load_pixbuf_fitting(pixbuf,
+					allocation.width,
+					allocation.height);
+
+			if (textview->stop_loading) {
+				return;
+			}
+
 			uri_str = g_filename_to_uri(filename, NULL, NULL);
 			if (uri_str) {
 				uri = g_new0(ClickableText, 1);
@@ -729,7 +793,10 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
 				uri->start = gtk_text_iter_get_offset(&iter);
 				
 				gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
-				
+				if (textview->stop_loading) {
+					g_free(uri);
+					return;
+				}
 				uri->end = uri->start + 1;
 				uri->filename = procmime_get_part_file_name(mimeinfo);
 				textview->uri_list =
@@ -741,6 +808,9 @@ static void textview_add_part(TextView *textview, MimeInfo *mimeinfo)
 						&start_iter, &iter);
 			} else {
 				gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
+				if (textview->stop_loading) {
+					return;
+				}
 				gtk_text_buffer_insert(buffer, &iter, " ", 1);
 			}
 
@@ -846,21 +916,45 @@ void textview_show_error(TextView *textview)
 		      "  This is probably due to a network error.\n"
 		      "\n"
 		      "  Use "));
-	TEXTVIEW_INSERT_LINK(_("'View Log'"), "sc://view_log", NULL);
+	TEXTVIEW_INSERT_LINK(_("'Network Log'"), "sc://view_log", NULL);
 	TEXTVIEW_INSERT(_(" in the Tools menu for more information."));
 	textview_show_icon(textview, GTK_STOCK_DIALOG_ERROR);
 }
 
+void textview_show_info(TextView *textview, const gchar *info_str)
+{
+	GtkTextView *text;
+	GtkTextBuffer *buffer;
+	GtkTextIter iter;
+
+	textview_set_font(textview, NULL);
+	textview_clear(textview);
+
+	text = GTK_TEXT_VIEW(textview->text);
+	buffer = gtk_text_view_get_buffer(text);
+	gtk_text_buffer_get_start_iter(buffer, &iter);
+
+	TEXTVIEW_INSERT(info_str);
+	textview_show_icon(textview, GTK_STOCK_DIALOG_INFO);
+	textview_cursor_normal(textview);
+}
+
 void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 {
 	GtkTextView *text;
 	GtkTextBuffer *buffer;
 	GtkTextIter iter;
 	const gchar *name;
-	gchar *content_type;
+	gchar *content_type, *shortcut;
+	GtkUIManager *ui_manager;
 
 	if (!partinfo) return;
 
+	if (textview->messageview->window != NULL)
+		ui_manager = textview->messageview->ui_manager;
+	else
+		ui_manager = textview->messageview->mainwin->ui_manager;
+
 	textview_set_font(textview, NULL);
 	textview_clear(textview);
 
@@ -895,7 +989,11 @@ void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 	TEXTVIEW_INSERT(_("     - To save, select "));
 	TEXTVIEW_INSERT_LINK(_("'Save as...'"), "sc://save_as", NULL);
 #ifndef GENERIC_UMPC
-	TEXTVIEW_INSERT(_(" (Shortcut key: 'y')"));
+	TEXTVIEW_INSERT(_(" (Shortcut key: '"));
+	shortcut = cm_menu_item_get_shortcut(ui_manager, "Menu/File/SavePartAs");
+	TEXTVIEW_INSERT(shortcut);
+	g_free(shortcut);
+	TEXTVIEW_INSERT("')");
 #endif
 	TEXTVIEW_INSERT("\n");
 
@@ -903,7 +1001,11 @@ void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 	TEXTVIEW_INSERT_LINK(_("'Display as text'"), "sc://display_as_text", NULL);
 
 #ifndef GENERIC_UMPC
-	TEXTVIEW_INSERT(_(" (Shortcut key: 't')"));
+	TEXTVIEW_INSERT(_(" (Shortcut key: '"));
+	shortcut = cm_menu_item_get_shortcut(ui_manager, "Menu/View/Part/AsText");
+	TEXTVIEW_INSERT(shortcut);
+	g_free(shortcut);
+	TEXTVIEW_INSERT("')");
 #endif
 	TEXTVIEW_INSERT("\n");
 
@@ -911,13 +1013,21 @@ void textview_show_mime_part(TextView *textview, MimeInfo *partinfo)
 	TEXTVIEW_INSERT_LINK(_("'Open'"), "sc://open", NULL);
 
 #ifndef GENERIC_UMPC
-	TEXTVIEW_INSERT(_(" (Shortcut key: 'l')\n"));
+	TEXTVIEW_INSERT(_(" (Shortcut key: '"));
+	shortcut = cm_menu_item_get_shortcut(ui_manager, "Menu/View/Part/Open");
+	TEXTVIEW_INSERT(shortcut);
+	g_free(shortcut);
+	TEXTVIEW_INSERT("')\n");
 	TEXTVIEW_INSERT(_("       (alternately double-click, or click the middle "));
 	TEXTVIEW_INSERT(_("mouse button)\n"));
 #ifndef G_OS_WIN32
 	TEXTVIEW_INSERT(_("     - Or use "));
 	TEXTVIEW_INSERT_LINK(_("'Open with...'"), "sc://open_with", NULL);
-	TEXTVIEW_INSERT(_(" (Shortcut key: 'o')"));
+	TEXTVIEW_INSERT(_(" (Shortcut key: '"));
+	shortcut = cm_menu_item_get_shortcut(ui_manager, "Menu/View/Part/OpenWith");
+	TEXTVIEW_INSERT(shortcut);
+	g_free(shortcut);
+	TEXTVIEW_INSERT("')");
 #endif
 #endif
 	TEXTVIEW_INSERT("\n");
@@ -957,6 +1067,7 @@ static void textview_write_body(TextView *textview, MimeInfo *mimeinfo)
 	procmime_force_encoding(textview->messageview->forced_encoding);
 	
 	textview->is_in_signature = FALSE;
+	textview->is_diff = FALSE;
 
 	procmime_decode_content(mimeinfo);
 
@@ -1046,6 +1157,10 @@ static void textview_write_body(TextView *textview, MimeInfo *mimeinfo)
 #endif
 	} else {
 textview_default:
+		if (!g_ascii_strcasecmp(mimeinfo->subtype, "x-patch")
+				|| !g_ascii_strcasecmp(mimeinfo->subtype, "x-diff"))
+			textview->is_diff = TRUE;
+
 		if (mimeinfo->content == MIMECONTENT_MEM)
 			tmpfp = str_open_as_stream(mimeinfo->data.mem);
 		else
@@ -1246,7 +1361,7 @@ static void textview_make_clickable_parts(TextView *textview,
 	gtk_text_buffer_get_end_iter(buffer, &iter);
 
 	/* parse for clickable parts, and build a list of begin and end positions  */
-	for (walk = mybuf, n = 0;;) {
+	for (walk = mybuf;;) {
 		gint last_index = PARSE_ELEMS;
 		gchar *scanpos = NULL;
 
@@ -1366,7 +1481,7 @@ static void textview_make_clickable_parts_later(TextView *textview,
 	offset = gtk_text_iter_get_offset(&start_iter);
 
 	/* parse for clickable parts, and build a list of begin and end positions  */
-	for (walk = mybuf, n = 0;;) {
+	for (walk = mybuf;;) {
 		gint last_index = PARSE_ELEMS;
 		gchar *scanpos = NULL;
 
@@ -1487,10 +1602,25 @@ static void textview_write_line(TextView *textview, const gchar *str,
 		fg_color = quote_tag_str;
 	}
 
-	if (prefs_common.enable_color && 
-	    (strcmp(buf,"-- \n") == 0 || strcmp(buf, "- -- \n") == 0 || textview->is_in_signature)) {
-		fg_color = "signature";
-		textview->is_in_signature = TRUE;
+	if (prefs_common.enable_color) {
+		if (textview->is_diff) {
+			if (strncmp(buf, "+++ ", 4) == 0)
+				fg_color = "diff-add-file";
+			else if (buf[0] == '+')
+				fg_color = "diff-add";
+			else if (strncmp(buf, "--- ", 4) == 0)
+				fg_color = "diff-del-file";
+			else if (buf[0] == '-')
+				fg_color = "diff-del";
+			else if (strncmp(buf, "@@ ", 3) == 0 &&
+					strcmp(buf+strlen(buf)-4, " @@\n") == 0)
+				fg_color = "diff-hunk";
+		} else if (strcmp(buf,"-- \n") == 0
+				|| strcmp(buf, "- -- \n") == 0
+				|| textview->is_in_signature) {
+			fg_color = "signature";
+			textview->is_in_signature = TRUE;
+		}
 	}
 
 	if (real_quotelevel > -1 && do_quote_folding) {
@@ -1507,6 +1637,7 @@ do_quote:
 			uri = g_new0(ClickableText, 1);
 			uri->uri = g_strdup("");
 			uri->data = g_strdup(buf);
+			uri->data_len = strlen(uri->data);
 			uri->start = gtk_text_iter_get_offset(&iter);
 			uri->is_quote = TRUE;
 			uri->quote_level = real_quotelevel;
@@ -1539,11 +1670,12 @@ do_quote:
 					textview->prev_quote_level = -1;
 					goto do_quote;
 				}
-				e_len = lasturi->data ? strlen(lasturi->data):0;
+				e_len = lasturi->data ? lasturi->data_len:0;
 				n_len = strlen(buf);
 				lasturi->data = g_realloc((gchar *)lasturi->data, e_len + n_len + 1);
 				strcpy((gchar *)lasturi->data + e_len, buf);
 				*((gchar *)lasturi->data + e_len + n_len) = '\0';
+				lasturi->data_len += n_len;
 			}
 		}
 	} else {
@@ -1555,7 +1687,6 @@ do_quote:
 void textview_write_link(TextView *textview, const gchar *str,
 			 const gchar *uri, CodeConverter *conv)
 {
-    	GdkColor *link_color = NULL;
 	GtkTextView *text;
 	GtkTextBuffer *buffer;
 	GtkTextIter iter;
@@ -1596,9 +1727,6 @@ void textview_write_link(TextView *textview, const gchar *str,
 	if (bufp > buf)
 		gtk_text_buffer_insert(buffer, &iter, buf, bufp - buf);
 
-    	if (prefs_common.enable_color) {
-		link_color = &uri_color;
-    	}
 	r_uri = g_new0(ClickableText, 1);
 	r_uri->uri = g_strdup(uri);
 	r_uri->start = gtk_text_iter_get_offset(&iter);
@@ -1658,11 +1786,6 @@ void textview_destroy(TextView *textview)
 	g_free(textview);
 }
 
-void textview_set_all_headers(TextView *textview, gboolean all_headers)
-{
-	textview->show_all_headers = all_headers;
-}
-
 #define CHANGE_TAG_FONT(tagname, font) { \
 	tag = gtk_text_tag_table_lookup(tags, tagname); \
 	if (tag) \
@@ -1784,7 +1907,7 @@ static GPtrArray *textview_scan_header(TextView *textview, FILE *fp)
 
 	cm_return_val_if_fail(fp != NULL, NULL);
 
-	if (textview->show_all_headers) {
+	if (prefs_common.show_all_headers) {
 		headers = procheader_get_header_array_asis(fp);
 		sorted_headers = g_ptr_array_new();
 		for (i = 0; i < headers->len; i++) {
@@ -1848,6 +1971,7 @@ static GPtrArray *textview_scan_header(TextView *textview, FILE *fp)
 
 static void textview_show_face(TextView *textview)
 {
+	GtkAllocation allocation;
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
 	MsgInfo *msginfo = textview->messageview->msginfo;
 	int x = 0;
@@ -1868,7 +1992,8 @@ static void textview_show_face(TextView *textview)
 
 	gtk_widget_show(textview->image);
 	
-	x = textview->text->allocation.width - WIDTH -5;
+	gtk_widget_get_allocation(textview->text, &allocation);
+	x = allocation.width - WIDTH -5;
 
 	gtk_text_view_add_child_in_window(text, textview->image, 
 		GTK_TEXT_WINDOW_TEXT, x, 5);
@@ -1885,6 +2010,7 @@ bail:
 
 void textview_show_icon(TextView *textview, const gchar *stock_id)
 {
+	GtkAllocation allocation;
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
 	int x = 0;
 	
@@ -1896,7 +2022,8 @@ void textview_show_icon(TextView *textview, const gchar *stock_id)
 
 	gtk_widget_show(textview->image);
 	
-	x = textview->text->allocation.width - WIDTH -5;
+	gtk_widget_get_allocation(textview->text, &allocation);
+	x = allocation.width - WIDTH -5;
 
 	gtk_text_view_add_child_in_window(text, textview->image, 
 		GTK_TEXT_WINDOW_TEXT, x, 5);
@@ -1910,6 +2037,7 @@ void textview_show_icon(TextView *textview, const gchar *stock_id)
 #if HAVE_LIBCOMPFACE
 static void textview_show_xface(TextView *textview)
 {
+	GtkAllocation allocation;
 	MsgInfo *msginfo = textview->messageview->msginfo;
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
 	int x = 0;
@@ -1942,7 +2070,8 @@ static void textview_show_xface(TextView *textview)
 
 	gtk_widget_show(textview->image);
 	
-	x = textview->text->allocation.width - WIDTH -5;
+	gtk_widget_get_allocation(textview->text, &allocation);
+	x = allocation.width - WIDTH -5;
 
 	gtk_text_view_add_child_in_window(text, textview->image, 
 		GTK_TEXT_WINDOW_TEXT, x, 5);
@@ -1960,6 +2089,7 @@ bail:
 
 static void textview_save_contact_pic(TextView *textview)
 {
+#ifndef USE_NEW_ADDRBOOK
 	MsgInfo *msginfo = textview->messageview->msginfo;
 	gchar *filename = NULL;
 	GError *error = NULL;
@@ -1983,10 +2113,14 @@ static void textview_save_contact_pic(TextView *textview)
 		}
 	}
 	g_free(filename);
+#else
+	/* new address book */
+#endif
 }
 
 static void textview_show_contact_pic(TextView *textview)
 {
+#ifndef USE_NEW_ADDRBOOK
 	MsgInfo *msginfo = textview->messageview->msginfo;
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
 	int x = 0;
@@ -1994,6 +2128,7 @@ static void textview_show_contact_pic(TextView *textview)
 	GError *error = NULL;
 	GdkPixbuf *picture = NULL;
 	gint w, h;
+	GtkAllocation allocation;
 				
 	if (prefs_common.display_header_pane
 	||  !prefs_common.display_xface)
@@ -2038,7 +2173,8 @@ static void textview_show_contact_pic(TextView *textview)
 
 	gtk_widget_show(textview->image);
 	
-	x = textview->text->allocation.width - WIDTH -5;
+	gtk_widget_get_allocation(textview->text, &allocation);
+	x = allocation.width - WIDTH -5;
 
 	gtk_text_view_add_child_in_window(text, textview->image, 
 		GTK_TEXT_WINDOW_TEXT, x, 5);
@@ -2050,7 +2186,9 @@ bail:
 	if (textview->image) 
 		gtk_widget_destroy(textview->image);
 	textview->image = NULL;
-	
+#else
+	/* new address book */
+#endif	
 }
 
 static gint textview_tag_cmp_list(gconstpointer a, gconstpointer b)
@@ -2168,12 +2306,20 @@ static void textview_show_header(TextView *textview, GPtrArray *headers)
 		    procheader_headername_equal(header->name, "To")      ||
 		    procheader_headername_equal(header->name, "Cc"))
 			unfold_line(header->body);
-
-		if ((procheader_headername_equal(header->name, "X-Mailer") ||
-		     procheader_headername_equal(header->name,
+		
+		if (procheader_headername_equal(header->name, "Date") &&
+		    prefs_common.msgview_date_format) {
+			gchar hbody[80];
+			
+			procheader_date_parse(hbody, header->body, sizeof(hbody));
+			gtk_text_buffer_get_end_iter (buffer, &iter);
+			gtk_text_buffer_insert_with_tags_by_name
+				(buffer, &iter, hbody, -1, "header", NULL);
+		} else if ((procheader_headername_equal(header->name, "X-Mailer") ||
+				procheader_headername_equal(header->name,
 						 "X-Newsreader")) &&
-		    (strstr(header->body, "Claws Mail") != NULL ||
-		     strstr(header->body, "Sylpheed-Claws") != NULL)) {
+				(strstr(header->body, "Claws Mail") != NULL ||
+				strstr(header->body, "Sylpheed-Claws") != NULL)) {
 			gtk_text_buffer_get_end_iter (buffer, &iter);
 			gtk_text_buffer_insert_with_tags_by_name
 				(buffer, &iter, header->body, -1,
@@ -2222,7 +2368,7 @@ gboolean textview_search_string_backward(TextView *textview, const gchar *str,
 void textview_scroll_one_line(TextView *textview, gboolean up)
 {
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
-	GtkAdjustment *vadj = text->vadjustment;
+	GtkAdjustment *vadj = gtk_text_view_get_vadjustment(text);
 
 	gtkutils_scroll_one_line(GTK_WIDGET(text), vadj, up);
 }
@@ -2230,11 +2376,28 @@ void textview_scroll_one_line(TextView *textview, gboolean up)
 gboolean textview_scroll_page(TextView *textview, gboolean up)
 {
 	GtkTextView *text = GTK_TEXT_VIEW(textview->text);
-	GtkAdjustment *vadj = text->vadjustment;
+	GtkAdjustment *vadj = gtk_text_view_get_vadjustment(text);
 
 	return gtkutils_scroll_page(GTK_WIDGET(text), vadj, up);
 }
 
+void textview_scroll_max(TextView *textview, gboolean up)
+{
+	GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview->text));
+	GtkTextIter iter;
+	
+	if (up) {
+		gtk_text_buffer_get_start_iter(buffer, &iter);
+		gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textview->text),
+						&iter, 0.0, TRUE, 0.0, 1.0);
+	
+	} else {
+		gtk_text_buffer_get_end_iter(buffer, &iter);
+		gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textview->text),
+						&iter, 0.0, TRUE, 0.0, 0.0);
+	}
+}
+
 #define KEY_PRESS_EVENT_STOP() \
 	g_signal_stop_emission_by_name(G_OBJECT(widget), \
 				       "key_press_event");
@@ -2242,6 +2405,7 @@ gboolean textview_scroll_page(TextView *textview, gboolean up)
 static gint textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 				 TextView *textview)
 {
+	GdkWindow *window;
 	SummaryView *summaryview = NULL;
 	MessageView *messageview = textview->messageview;
 
@@ -2250,19 +2414,21 @@ static gint textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 		summaryview = messageview->mainwin->summaryview;
 
 	switch (event->keyval) {
-	case GDK_Tab:
-	case GDK_Home:
-	case GDK_Left:
-	case GDK_Up:
-	case GDK_Right:
-	case GDK_Down:
-	case GDK_Page_Up:
-	case GDK_Page_Down:
-	case GDK_End:
-	case GDK_Control_L:
-	case GDK_Control_R:
+	case GDK_KEY_Tab:
+	case GDK_KEY_Left:
+	case GDK_KEY_Up:
+	case GDK_KEY_Right:
+	case GDK_KEY_Down:
+	case GDK_KEY_Page_Up:
+	case GDK_KEY_Page_Down:
+	case GDK_KEY_Control_L:
+	case GDK_KEY_Control_R:
 		return FALSE;
-	case GDK_space:
+	case GDK_KEY_Home:
+	case GDK_KEY_End:
+		textview_scroll_max(textview,(event->keyval == GDK_KEY_Home));
+		return TRUE;
+	case GDK_KEY_space:
 		if (summaryview)
 			summary_pass_key_press_event(summaryview, event);
 		else
@@ -2271,38 +2437,26 @@ static gint textview_key_pressed(GtkWidget *widget, GdkEventKey *event,
 				 (event->state &
 				  (GDK_SHIFT_MASK|GDK_MOD1_MASK)) != 0);
 		break;
-	case GDK_BackSpace:
+	case GDK_KEY_BackSpace:
 		mimeview_scroll_page(messageview->mimeview, TRUE);
 		break;
-	case GDK_Return:
-	case GDK_KP_Enter:
+	case GDK_KEY_Return:
+	case GDK_KEY_KP_Enter:
 		mimeview_scroll_one_line
 			(messageview->mimeview, (event->state &
 				    (GDK_SHIFT_MASK|GDK_MOD1_MASK)) != 0);
 		break;
-	case GDK_Delete:
+	case GDK_KEY_Delete:
 		if (summaryview)
 			summary_pass_key_press_event(summaryview, event);
 		break;
-	case GDK_y:
-	case GDK_t:
-	case GDK_l:
-	case GDK_o:
-	case GDK_c:
-	case GDK_a:
-		if ((event->state & (GDK_MOD1_MASK|GDK_CONTROL_MASK)) == 0) {
-			KEY_PRESS_EVENT_STOP();
-			mimeview_pass_key_press_event(messageview->mimeview,
-						      event);
-			break;
-		}
-		/* possible fall through */
 	default:
+		window = gtk_widget_get_window(messageview->mainwin->window);
 		if (summaryview &&
-		    event->window != messageview->mainwin->window->window) {
+		    event->window != window) {
 			GdkEventKey tmpev = *event;
 
-			tmpev.window = messageview->mainwin->window->window;
+			tmpev.window = window;
 			KEY_PRESS_EVENT_STOP();
 			gtk_widget_event(messageview->mainwin->window,
 					 (GdkEvent *)&tmpev);
@@ -2320,7 +2474,7 @@ static gboolean textview_motion_notify(GtkWidget *widget,
 	if (textview->loading)
 		return FALSE;
 	textview_uri_update(textview, event->x, event->y);
-	gdk_window_get_pointer(widget->window, NULL, NULL, NULL);
+	gdk_window_get_pointer(gtk_widget_get_window(widget), NULL, NULL, NULL);
 
 	return FALSE;
 }
@@ -2353,7 +2507,7 @@ static gboolean textview_visibility_notify(GtkWidget *widget,
 	if (window != event->window)
 		return FALSE;
 	
-	gdk_window_get_pointer(widget->window, &wx, &wy, NULL);
+	gdk_window_get_pointer(gtk_widget_get_window(widget), &wx, &wy, NULL);
 	textview_uri_update(textview, wx, wy);
 
 	return FALSE;
@@ -2917,6 +3071,8 @@ static void save_file_cb (GtkAction *action, TextView *textview)
 	gchar *filepath = NULL;
 	gchar *filedir = NULL;
 	gchar *tmp_filename = NULL;
+	GtkWidget *window;
+
 	if (uri == NULL)
 		return;
 
@@ -2941,6 +3097,14 @@ static void save_file_cb (GtkAction *action, TextView *textview)
 
 	g_free(filename);
 
+	/* Pick correct window to set the file dialog "transient for" */
+	if (textview->messageview->window != NULL)
+		window = textview->messageview->window;
+	else
+		window = textview->messageview->mainwin->window;
+
+	manage_window_focus_in(window, NULL, NULL);
+
 	filename = filesel_select_file_save(_("Save as"), filepath);
 	if (!filename) {
 		g_free(filepath);
@@ -3005,6 +3169,7 @@ static void add_uri_to_addrbook_cb (GtkAction *action, TextView *textview)
 	GtkWidget *image = NULL;
 	GdkPixbuf *picture = NULL;
 	gboolean use_picture = FALSE;
+
 	if (uri == NULL)
 		return;
 
@@ -3037,7 +3202,13 @@ static void add_uri_to_addrbook_cb (GtkAction *action, TextView *textview)
 	if (image)
 		picture = gtk_image_get_pixbuf(GTK_IMAGE(image));
 
+#ifndef USE_NEW_ADDRBOOK
 	addressbook_add_contact( fromname, fromaddress, NULL, picture);
+#else
+	if (addressadd_selection(fromname, fromaddress, NULL, picture)) {
+		debug_print( "addressbook_add_contact - added\n" );
+	}
+#endif
 
 	g_free(fromaddress);
 	g_free(fromname);
