@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 2001-2012 Hiroyuki Yamamoto and the Claws Mail team
+ * Copyright (C) 2001-2013 Hiroyuki Yamamoto and the Claws Mail team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,6 +59,7 @@
 #include "alertpanel.h"
 #include "imap.h"
 #include "account.h"
+#include "send_message.h"
 #ifndef USE_NEW_ADDRBOOK
 	#include "addressbook.h"
 #else
@@ -241,7 +242,7 @@ struct {
 struct {
 	const gchar  *conf_file;
 	GSList       *item_list;
-} toolbar_config[3] = {
+} toolbar_config[NUM_TOOLBARS] = {
 	{ "toolbar_main.xml",    NULL},
 	{ "toolbar_compose.xml", NULL}, 
   	{ "toolbar_msgview.xml", NULL}
@@ -766,7 +767,7 @@ void toolbar_read_config_file(ToolbarType source)
 		else if (source == TOOLBAR_MSGVIEW) 
 			toolbar_set_default(TOOLBAR_MSGVIEW);
 		else {		
-			g_warning("failed to write Toolbar Configuration to %s\n", toolbar_config[source].conf_file);
+			g_warning("refusing to write unknown Toolbar Configuration number %d\n", source);
 			return;
 		}
 
@@ -2223,18 +2224,9 @@ Toolbar *toolbar_create(ToolbarType 	 type,
 		activate_learn_button(toolbar_data, prefs_common.toolbar_style,
 				LEARN_SPAM);
 	
-#ifndef MAEMO
 	gtk_container_add(GTK_CONTAINER(container), toolbar);
 	gtk_container_set_border_width(GTK_CONTAINER(container), 0);
-#else
-	if ( GTK_IS_WINDOW(container) ) {
-		hildon_window_add_toolbar (HILDON_WINDOW(container), GTK_TOOLBAR(toolbar));
-		gtk_widget_show_all (container);
-	} else {
-		gtk_container_add(GTK_CONTAINER(container), toolbar);
-		gtk_container_set_border_width(GTK_CONTAINER(container), 2);
-	}
-#endif
+
 	return toolbar_data; 
 }
 
@@ -2295,9 +2287,6 @@ void toolbar_update(ToolbarType type, gpointer data)
 	default:
 		return;
 	}
-#ifdef MAEMO
-	hildon_window_remove_toolbar(HILDON_WINDOW(handlebox), GTK_TOOLBAR(toolbar_data->toolbar));
-#endif
 	toolbar_init(toolbar_data);
  	toolbar_data = toolbar_create(type, handlebox, data);
 #endif
